package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.params.NaccacheSternKeyParameters;
import cn.com.infosec.crypto.params.NaccacheSternPrivateKeyParameters;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.util.Arrays;
import java.io.PrintStream;
import java.math.BigInteger;
import java.util.Vector;

public class NaccacheSternEngine implements AsymmetricBlockCipher {
   private static BigInteger ZERO = BigInteger.valueOf(0L);
   private static BigInteger ONE = BigInteger.valueOf(1L);
   private boolean forEncryption;
   private NaccacheSternKeyParameters key;
   private Vector[] lookup = null;
   private boolean debug = false;

   private static BigInteger chineseRemainder(Vector congruences, Vector primes) {
      BigInteger var2 = ZERO;
      BigInteger var3 = ONE;

      int var4;
      for(var4 = 0; var4 < primes.size(); ++var4) {
         var3 = var3.multiply((BigInteger)primes.elementAt(var4));
      }

      for(var4 = 0; var4 < primes.size(); ++var4) {
         BigInteger var10000 = var2;
         BigInteger var10001 = var3.divide(var2 = (BigInteger)primes.elementAt(var4));
         var2 = var10000.add(var10001.multiply(var10001.modInverse(var2)).multiply((BigInteger)congruences.elementAt(var4)));
      }

      return var2.mod(var3);
   }

   public void init(boolean forEncryption, CipherParameters param) {
      this.forEncryption = forEncryption;
      if (param instanceof ParametersWithRandom) {
         param = ((ParametersWithRandom)param).getParameters();
      }

      this.key = (NaccacheSternKeyParameters)param;
      if (!this.forEncryption) {
         if (this.debug) {
            System.out.println("Constructing lookup Array");
         }

         NaccacheSternPrivateKeyParameters forEncryption1;
         Vector param1;
         this.lookup = new Vector[(param1 = (forEncryption1 = (NaccacheSternPrivateKeyParameters)this.key).getSmallPrimes()).size()];

         for(int var3 = 0; var3 < param1.size(); ++var3) {
            BigInteger var4;
            int var5 = (var4 = (BigInteger)param1.elementAt(var3)).intValue();
            Vector[] var10002 = this.lookup;
            Vector var6;
            var6 = new Vector.<init>();
            var10002[var3] = var6;
            this.lookup[var3].addElement(ONE);
            if (this.debug) {
               System.out.println("Constructing lookup ArrayList for " + var5);
            }

            BigInteger var11 = ZERO;

            for(int var7 = 1; var7 < var5; ++var7) {
               BigInteger var8 = (var11 = var11.add(forEncryption1.getPhi_n())).divide(var4);
               this.lookup[var3].addElement(forEncryption1.getG().modPow(var8, forEncryption1.getModulus()));
            }
         }
      }

   }

   public void setDebug(boolean debug) {
      this.debug = debug;
   }

   public int getInputBlockSize() {
      return this.forEncryption ? (this.key.getLowerSigmaBound() + 7) / 8 - 1 : this.key.getModulus().toByteArray().length;
   }

   public int getOutputBlockSize() {
      return this.forEncryption ? this.key.getModulus().toByteArray().length : (this.key.getLowerSigmaBound() + 7) / 8 - 1;
   }

   public byte[] processBlock(byte[] in, int inOff, int len) {
      if (this.key == null) {
         throw new IllegalStateException("NaccacheStern engine not initialised");
      } else if (len <= this.getInputBlockSize() + 1) {
         if (!this.forEncryption && len < this.getInputBlockSize()) {
            throw new InvalidCipherTextException("BlockLength does not match modulus for Naccache-Stern cipher.\n");
         } else {
            byte[] var10000;
            if (inOff != 0 || len != in.length) {
               var10000 = in;
               in = new byte[len];
               System.arraycopy(var10000, inOff, in, 0, len);
            }

            BigInteger inOff1;
            inOff1 = new BigInteger.<init>(1, in);
            if (this.debug) {
               System.out.println("input as BigInteger: " + inOff1);
            }

            if (this.forEncryption) {
               var10000 = this.encrypt(inOff1);
            } else {
               Vector in1;
               in1 = new Vector.<init>();
               NaccacheSternPrivateKeyParameters len1;
               Vector var4 = (len1 = (NaccacheSternPrivateKeyParameters)this.key).getSmallPrimes();

               for(int var5 = 0; var5 < var4.size(); ++var5) {
                  BigInteger var6 = len1.getPhi_n().divide((BigInteger)var4.elementAt(var5));
                  var6 = inOff1.modPow(var6, len1.getModulus());
                  Vector[] var14 = this.lookup;
                  Vector var7 = var14[var5];
                  StringBuilder in2;
                  PrintStream var15;
                  StringBuilder var10001;
                  if (var14[var5].size() != ((BigInteger)var4.elementAt(var5)).intValue()) {
                     if (this.debug) {
                        var15 = System.out;
                        var10001 = in2 = new StringBuilder;
                        in2.<init>("Prime is ");
                        var15.println(var10001.append(var4.elementAt(var5)).append(", lookup table has size ").append(var7.size()).toString());
                     }

                     StringBuilder var10002 = in2 = new StringBuilder;
                     in2.<init>("Error in lookup Array for ");
                     throw new InvalidCipherTextException(var10002.append(((BigInteger)var4.elementAt(var5)).intValue()).append(": Size mismatch. Expected ArrayList with length ").append(((BigInteger)var4.elementAt(var5)).intValue()).append(" but found ArrayList of length ").append(this.lookup[var5].size()).toString());
                  }

                  int var13;
                  if ((var13 = var7.indexOf(var6)) == -1) {
                     if (this.debug) {
                        var15 = System.out;
                        var10001 = in2 = new StringBuilder;
                        in2.<init>("Actual prime is ");
                        var15.println(var10001.append(var4.elementAt(var5)).toString());
                        System.out.println("Decrypted value is " + var6);
                        var15 = System.out;
                        var10001 = in2 = new StringBuilder;
                        in2.<init>("LookupList for ");
                        var15.println(var10001.append(var4.elementAt(var5)).append(" with size ").append(this.lookup[var5].size()).append(" is: ").toString());

                        for(int in3 = 0; in3 < this.lookup[var5].size(); ++in3) {
                           System.out.println(this.lookup[var5].elementAt(in3));
                        }
                     }

                     throw new InvalidCipherTextException("Lookup failed");
                  }

                  in1.addElement(BigInteger.valueOf((long)var13));
               }

               var10000 = chineseRemainder(in1, var4).toByteArray();
            }

            return var10000;
         }
      } else {
         throw new DataLengthException("input too large for Naccache-Stern cipher.\n");
      }
   }

   public byte[] encrypt(BigInteger plain) {
      NaccacheSternEngine var10000 = this;
      byte[] var2;
      Arrays.fill(var2 = this.key.getModulus().toByteArray(), (byte)0);
      byte[] var10001 = this.key.getG().modPow(plain, this.key.getModulus()).toByteArray();
      int var3 = var2.length - var10001.length;
      int plain1 = var10001.length;
      System.arraycopy(var10001, 0, var2, var3, plain1);
      if (var10000.debug) {
         System.out.println("Encrypted value is:  " + new BigInteger(var2));
      }

      return var2;
   }

   public byte[] addCryptedBlocks(byte[] block1, byte[] block2) {
      if (this.forEncryption) {
         if (block1.length > this.getOutputBlockSize() || block2.length > this.getOutputBlockSize()) {
            throw new InvalidCipherTextException("BlockLength too large for simple addition.\n");
         }
      } else if (block1.length > this.getInputBlockSize() || block2.length > this.getInputBlockSize()) {
         throw new InvalidCipherTextException("BlockLength too large for simple addition.\n");
      }

      BigInteger var3;
      BigInteger var10001 = var3 = new BigInteger;
      var10001.<init>(1, block1);
      BigInteger block11;
      BigInteger var10002 = block11 = new BigInteger;
      var10002.<init>(1, block2);
      BigInteger block21 = var10001.multiply(var10002).mod(this.key.getModulus());
      if (this.debug) {
         System.out.println("c(m1) as BigInteger:....... " + var3);
         System.out.println("c(m2) as BigInteger:....... " + block11);
         System.out.println("c(m1)*c(m2)%n = c(m1+m2)%n: " + block21);
      }

      byte[] var4;
      byte[] var10000 = var4 = this.key.getModulus().toByteArray();
      Arrays.fill(var4, (byte)0);
      byte[] var9 = block21.toByteArray();
      int block12 = var4.length - block21.toByteArray().length;
      int block22 = block21.toByteArray().length;
      System.arraycopy(var9, 0, var4, block12, block22);
      return var10000;
   }

   public byte[] processData(byte[] data) {
      if (this.debug) {
         System.out.println();
      }

      if (data.length > this.getInputBlockSize()) {
         int var2 = this.getInputBlockSize();
         int var3 = this.getOutputBlockSize();
         if (this.debug) {
            System.out.println("Input blocksize is:  " + var2 + " bytes");
            System.out.println("Output blocksize is: " + var3 + " bytes");
            System.out.println("Data has length:.... " + data.length + " bytes");
         }

         int var4 = 0;
         int var5 = 0;

         byte[] var8;
         byte[] var10001;
         for(var8 = new byte[(data.length / var2 + 1) * var3]; var4 < data.length; var5 += var10001.length) {
            int var6;
            byte[] var10;
            if ((var6 = var4 + var2) < data.length) {
               byte[] var9 = this.processBlock(data, var4, var2);
               int var10000 = var6;
               var10 = var9;
               var4 = var10000;
            } else {
               var10 = this.processBlock(data, var4, data.length - var4);
               var4 += data.length - var4;
            }

            if (this.debug) {
               System.out.println("new datapos is " + var4);
            }

            if (var10 == null) {
               if (this.debug) {
                  System.out.println("cipher returned null");
               }

               throw new InvalidCipherTextException("cipher returned null");
            }

            var10001 = var10;
            byte[] var10002 = var10;
            var6 = var10.length;
            System.arraycopy(var10002, 0, var8, var5, var6);
         }

         NaccacheSternEngine var11 = this;
         byte[] var7 = new byte[var5];
         System.arraycopy(var8, 0, var7, 0, var5);
         if (var11.debug) {
            System.out.println("returning " + var5 + " bytes");
         }

         return var7;
      } else {
         if (this.debug) {
            System.out.println("data size is less then input block size, processing directly");
         }

         return this.processBlock(data, 0, data.length);
      }
   }
}
